@@ -53,7 +53,7 @@ display_add_error (GdmDisplay *d)
 {
 	if (errno != 0)
 		gdm_error (_("%s: Could not write new authorization entry: %s"),
-			   "add_auth_entry", g_strerror (errno));
+			   "add_auth_entry", strerror (errno));
 	else
 		gdm_error (_("%s: Could not write new authorization entry.  "
 			     "Possibly out of diskspace"),
@@ -63,7 +63,7 @@ display_add_error (GdmDisplay *d)
 			(_("GDM could not write a new authorization "
 			   "entry to disk.  Possibly out of diskspace.%s%s"),
 			 errno != 0 ? "  Error: " : "",
-			 errno != 0 ? g_strerror (errno) : "");
+			 errno != 0 ? strerror (errno) : "");
 		gdm_text_message_dialog (s);
 		g_free (s);
 	}
@@ -115,6 +115,11 @@ add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 
 	errno = 0;
 	if ( ! XauWriteAuth (af, xa)) {
+		free (xa->data);
+		free (xa->number);
+		free (xa->name);
+		free (xa->address);
+		free (xa);
 		display_add_error (d);
 		return FALSE;
 	}
@@ -122,6 +127,11 @@ add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 	if (af2 != NULL) {
 		errno = 0;
 		if ( ! XauWriteAuth (af2, xa)) {
+			free (xa->data);
+			free (xa->number);
+			free (xa->name);
+			free (xa->address);
+			free (xa);
 			display_add_error (d);
 			return FALSE;
 		}
@@ -145,8 +155,7 @@ gboolean
 gdm_auth_secure_display (GdmDisplay *d)
 {
     FILE *af, *af_gdm;
-    struct hostent *hentry = NULL;
-    struct in_addr *ia = NULL;
+    gboolean is_local = FALSE;
     const char lo[] = {127,0,0,1};
     guint i;
     const GList *local_addys = NULL;
@@ -197,11 +206,13 @@ gdm_auth_secure_display (GdmDisplay *d)
 	    /* Make another authfile since the greeter can't read the server/user
 	     * readable file */
 	    d->authfile_gdm = g_strconcat (GdmServAuthDir, "/", d->name, ".Xauth", NULL);
-	    unlink (d->authfile_gdm);
-
-	    af_gdm = fopen (d->authfile_gdm, "w");
+	    af_gdm = gdm_safe_fopen_w (d->authfile_gdm);
 
 	    if (af_gdm == NULL) {
+		    gdm_error (_("%s: Cannot safely open %s"),
+			       "gdm_auth_user_remove",
+			       d->authfile_gdm);
+
 		    g_free (d->authfile_gdm);
 		    d->authfile_gdm = NULL;
 		    g_free (d->authfile);
@@ -212,11 +223,13 @@ gdm_auth_secure_display (GdmDisplay *d)
     } else {
 	    /* gdm and xserver authfile can be the same, server will run as root */
 	    d->authfile = g_strconcat (GdmServAuthDir, "/", d->name, ".Xauth", NULL);
-	    unlink (d->authfile);
-
-	    af = fopen (d->authfile, "w");
+	    af = gdm_safe_fopen_w (d->authfile);
 
 	    if (af == NULL) {
+		    gdm_error (_("%s: Cannot safely open %s"),
+			       "gdm_auth_user_remove",
+			       d->authfile);
+
 		    g_free (d->authfile);
 		    d->authfile = NULL;
 		    return FALSE;
@@ -258,23 +271,22 @@ gdm_auth_secure_display (GdmDisplay *d)
 		    d->hostname = g_strdup (hostname);
 	    }
 	    local_addys = gdm_peek_local_address_list ();
-    } else  {
-	    /* Find FQDN or IP of display host */
-	    hentry = gethostbyname (d->hostname);
 
-	    if (hentry == NULL) {
-		    gdm_error ("gdm_auth_secure_display: Error getting hentry for %s", d->hostname);
-		    /* eeek, there will be nothing to add */
-		    return FALSE;
-	    } else {
-		    /* first addy, would be loopback in case of local */
-		    ia = (struct in_addr *) hentry->h_addr_list[0];
+	    is_local = TRUE;
+    } else  {
+	    is_local = FALSE;
+	    if (gdm_is_local_addr (&(d->addr)))
+		    is_local = TRUE;
+	    for (i = 0; ! is_local && i < d->addr_count; i++) {
+		    if (gdm_is_local_addr (&(d->addrs[i]))) {
+			    is_local = TRUE;
+			    break;
+		    }
 	    }
     }
 
     /* Local access also in case the host is very local */
-    if (SERVER_IS_LOCAL (d) ||
-	(ia != NULL && gdm_is_local_addr (ia))) {
+    if (is_local) {
 	    gdm_debug ("gdm_auth_secure_display: Setting up socket access");
 
 	    if ( ! add_auth_entry (d, af, af_gdm, FamilyLocal,
@@ -310,14 +322,22 @@ gdm_auth_secure_display (GdmDisplay *d)
     }
 
     gdm_debug ("gdm_auth_secure_display: Setting up network access");
+
+    if ( ! SERVER_IS_LOCAL (d)) {
+	    /* we should write out an entry for d->addr since
+	       possibly it is not in d->addrs */
+	    if ( ! add_auth_entry (d, af, af_gdm, FamilyInternet,
+				   (char *)&(d->addr), 4))
+		    return FALSE;
+    }
     
     /* Network access: Write out an authentication entry for each of
      * this host's official addresses */
-    for (i = 0 ; hentry != NULL && i < hentry->h_length ; i++) {
-	    ia = (struct in_addr *) hentry->h_addr_list[i];
+    for (i = 0; i < d->addr_count; i++) {
+	    struct in_addr *ia = &(d->addrs[i]);
 
-	    if (ia == NULL)
-		    break;
+	    if (memcmp (ia, &(d->addr), sizeof (struct in_addr)) == 0)
+		    continue;
 
 	    if ( ! add_auth_entry (d, af, af_gdm, FamilyInternet,
 				   (char *)&ia->s_addr, 4))
@@ -327,7 +347,7 @@ gdm_auth_secure_display (GdmDisplay *d)
     /* Network access: Write out an authentication entry for each of
      * this host's local addresses if any */
     for (; local_addys != NULL; local_addys = local_addys->next) {
-	    ia = (struct in_addr *) local_addys->data;
+	    struct in_addr *ia = (struct in_addr *) local_addys->data;
 
 	    if (ia == NULL)
 		    break;
@@ -347,7 +367,7 @@ gdm_auth_secure_display (GdmDisplay *d)
     fclose (af);
     if (af_gdm != NULL)
 	    fclose (af_gdm);
-    gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
+    ve_setenv ("XAUTHORITY", GDM_AUTHFILE (d), TRUE);
 
     if (GdmDebug)
 	    gdm_debug ("gdm_auth_secure_display: Setting up access for %s - %d entries", 
@@ -384,7 +404,7 @@ try_open_append (const char *file)
 gboolean
 gdm_auth_user_add (GdmDisplay *d, uid_t user, const char *homedir)
 {
-    const char *authdir;
+    char *authdir;
     gint authfd;
     FILE *af;
     GSList *auths = NULL;
@@ -397,16 +417,22 @@ gdm_auth_user_add (GdmDisplay *d, uid_t user, const char *homedir)
 
     gdm_debug ("gdm_auth_user_add: Adding cookie for %d", user);
 
+    /* Determine whether UserAuthDir is specified. Otherwise ~user is used */
+    if ( ! ve_string_empty (GdmUserAuthDir) &&
+	strcmp (GdmUserAuthDir, "~") != 0) {
+	    if (strncmp (GdmUserAuthDir, "~/", 2) == 0) {
+		    authdir = g_strconcat (homedir, &GdmUserAuthDir[1], NULL);
+	    } else {
+		    authdir = g_strdup (GdmUserAuthDir);
+	    }
+    } else {
+	    authdir = g_strdup (homedir);
+    }
+
 try_user_add_again:
 
     locked = FALSE;
 
-    /* Determine whether UserAuthDir is specified. Otherwise ~user is used */
-    if ( ! ve_string_empty (GdmUserAuthDir))
-	authdir = GdmUserAuthDir;
-    else
-	authdir = homedir;
-
     umask (077);
 
     if (authdir == NULL)
@@ -418,7 +444,7 @@ try_user_add_again:
     if (automatic_tmp_dir ||
 	authdir == NULL ||
 	! gdm_file_check ("gdm_auth_user_add", user, authdir, GdmUserAuthFile, 
-			  TRUE, GdmUserMaxFile, GdmRelaxPerms) ||
+			  TRUE, FALSE, GdmUserMaxFile, GdmRelaxPerms) ||
 	! try_open_append (d->userauth)) {
 
 	/* No go. Let's create a fallback file in GdmUserAuthFB (/tmp) */
@@ -428,12 +454,15 @@ try_user_add_again:
 	authfd = g_mkstemp (d->userauth);
 
 	if (authfd == -1) {
-	    gdm_error (_("gdm_auth_user_add: Could not open cookie file %s"), d->userauth);
+	    gdm_error (_("%s: Could not open cookie file %s"),
+		       "gdm_auth_user_add",
+		       d->userauth);
 	    g_free (d->userauth);
 	    d->userauth = NULL;
 
 	    umask (022);
 
+	    g_free (authdir);
 	    return FALSE;
 	}
 
@@ -444,7 +473,9 @@ try_user_add_again:
 
 	/* FIXME: Better implement my own locking. The libXau one is not kosher */
 	if (XauLockAuth (d->userauth, 3, 3, 0) != LOCK_SUCCESS) {
-	    gdm_error (_("gdm_auth_user_add: Could not lock cookie file %s"), d->userauth);
+	    gdm_error (_("%s: Could not lock cookie file %s"),
+		       "gdm_auth_user_add",
+		       d->userauth);
 	    g_free (d->userauth);
 	    d->userauth = NULL;
 
@@ -456,12 +487,14 @@ try_user_add_again:
 
 	locked = TRUE;
 
-	af = fopen (d->userauth, "a+");
+	af = gdm_safe_fopen_ap (d->userauth);
     }
 
     if (!af) {
 	/* Really no need to clean up here - this process is a goner anyway */
-	gdm_error (_("gdm_auth_user_add: Could not open cookie file %s"), d->userauth);
+	gdm_error (_("%s: Could not open cookie file %s"),
+		   "gdm_auth_user_add",
+		   d->userauth);
 	if (locked)
 		XauUnlockAuth (d->userauth);
 	g_free (d->userauth);
@@ -474,6 +507,8 @@ try_user_add_again:
 		goto try_user_add_again;
 	}
 
+	g_free (authdir);
+
 	return FALSE; 
     }
 
@@ -516,6 +551,7 @@ try_user_add_again:
 
     umask (022);
 
+    g_free (authdir);
     return ret;
 }
 
@@ -558,10 +594,12 @@ gdm_auth_user_remove (GdmDisplay *d, uid_t user)
      * to it. So we better play it safe... */
 
     if ( ! gdm_file_check ("gdm_auth_user_remove", user, authdir, authfile, 
-			   TRUE, GdmUserMaxFile, GdmRelaxPerms)) {
+			   TRUE, FALSE, GdmUserMaxFile, GdmRelaxPerms)) {
 	    g_free (authdir);
 	    g_free (authfile);
-	    gdm_error (_("gdm_auth_user_remove: Ignoring suspiciously looking cookie file %s"), d->userauth);
+	    gdm_error (_("%s: Ignoring suspiciously looking cookie file %s"),
+		       "gdm_auth_user_remove",
+		       d->userauth);
 
 	    return; 
     }
@@ -576,11 +614,15 @@ gdm_auth_user_remove (GdmDisplay *d, uid_t user)
 	return;
     }
 
-    af = fopen (d->userauth, "a+");
+    af = gdm_safe_fopen_ap (d->userauth);
 
     if (!af) {
 	XauUnlockAuth (d->userauth);
 
+	gdm_error (_("%s: Cannot safely open %s"),
+		   "gdm_auth_user_remove",
+		   d->userauth);
+
 	g_free (d->userauth);
 	d->userauth = NULL;
 
@@ -660,5 +702,11 @@ gdm_auth_purge (GdmDisplay *d, FILE *af)
     g_slist_free (keep);
 }
 
+void
+gdm_auth_set_local_auth (GdmDisplay *d)
+{
+	XSetAuthorization ((char *)"MIT-MAGIC-COOKIE-1", (int) strlen ("MIT-MAGIC-COOKIE-1"),
+			   (char *)d->bcookie, (int) 16);
+}
 
 /* EOF */
