@@ -221,6 +221,7 @@ gdm_socket_handler (GIOChannel *source,
 	}
 
 	unixchan = g_io_channel_unix_new (newconn->fd);
+	g_io_channel_set_encoding (unixchan, NULL, NULL);
 	g_io_channel_set_buffered (unixchan, FALSE);
 
 	newconn->source = g_io_add_watch_full
@@ -274,6 +275,7 @@ gdm_connection_open_unix (const char *sockname, mode_t mode)
 	conn->n_subconnections = 0;
 
 	unixchan = g_io_channel_unix_new (conn->fd);
+	g_io_channel_set_encoding (unixchan, NULL, NULL);
 	g_io_channel_set_buffered (unixchan, FALSE);
 
 	conn->source = g_io_add_watch_full
@@ -293,7 +295,7 @@ gdm_connection_open_fd (int fd)
 	GIOChannel *unixchan;
 	GdmConnection *conn;
 
-	g_return_val_if_fail (fd < 0, NULL);
+	g_return_val_if_fail (fd >= 0, NULL);
 
 	conn = g_new0 (GdmConnection, 1);
 	conn->close_level = 0;
@@ -307,16 +309,15 @@ gdm_connection_open_fd (int fd)
 	conn->n_subconnections = 0;
 
 	unixchan = g_io_channel_unix_new (conn->fd);
+	g_io_channel_set_encoding (unixchan, NULL, NULL);
 	g_io_channel_set_buffered (unixchan, FALSE);
 
 	conn->source = g_io_add_watch_full
 		(unixchan, G_PRIORITY_DEFAULT,
 		 G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
-		 gdm_socket_handler, conn, NULL);
+		 gdm_connection_handler, conn, NULL);
 	g_io_channel_unref (unixchan);
 
-	listen (fd, 5);
-
 	return conn;
 }
 
@@ -357,6 +358,7 @@ gdm_connection_open_fifo (const char *fifo, mode_t mode)
 	conn->n_subconnections = 0;
 
 	fifochan = g_io_channel_unix_new (conn->fd);
+	g_io_channel_set_encoding (fifochan, NULL, NULL);
 	g_io_channel_set_buffered (fifochan, FALSE);
 
 	conn->source = g_io_add_watch_full
