@@ -26,6 +26,8 @@
 #include <sys/wait.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 #include <fcntl.h>
 #include <netinet/in.h>
 #include <net/if.h>
@@ -33,6 +35,7 @@
 #include <netdb.h>
 #include <errno.h>
 #include <grp.h>
+#include <pwd.h>
 #include <sys/types.h>
 #ifdef HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>
@@ -42,14 +45,15 @@
 
 #include "gdm.h"
 #include "misc.h"
+#include "xdmcp.h"
 #include "slave.h"
 
 /* Configuration option variables */
 extern gchar *GdmPidFile;
-extern gchar *GdmLocalNoPasswordUsers;
 extern gboolean GdmDebug;
 extern GSList *displays;
 extern int gdm_xdmcpfd;
+extern gboolean GdmXdmcp;
 
 extern char **environ;
 
@@ -238,7 +242,7 @@ gdm_clearenv_no_lang (void)
 			envs = g_list_prepend (envs, g_strdup (env));
 	}
 
-	gnome_clearenv ();
+	ve_clearenv ();
 
 	for (li = envs; li != NULL; li = li->next) {
 		putenv (li->data);
@@ -263,7 +267,7 @@ gdm_clearenv (void)
 			envs = g_list_prepend (envs, g_strdup (env));
 	}
 
-	gnome_clearenv ();
+	ve_clearenv ();
 
 	for (li = envs; li != NULL; li = li->next) {
 		putenv (li->data);
@@ -295,7 +299,7 @@ gdm_restoreenv (void)
 {
 	GList *li;
 
-	gnome_clearenv ();
+	ve_clearenv ();
 
 	for (li = stored_env; li != NULL; li = li->next) {
 		putenv (g_strdup (li->data));
@@ -319,6 +323,7 @@ gdm_get_free_display (int start, uid_t server_uid)
 		GSList *li;
 		struct stat s;
 		char buf[256];
+		FILE *fp;
 
 		for (li = displays; li != NULL; li = li->next) {
 			GdmDisplay *dsp = li->data;
@@ -344,6 +349,32 @@ gdm_get_free_display (int start, uid_t server_uid)
 		}
 		close (sock);
 
+		/* if lock file exists and the process exists */
+		g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", i);
+		if (stat (buf, &s) == 0 &&
+		    ! S_ISREG (s.st_mode)) {
+			/* Eeeek! not a regular file?  Perhaps someone
+			   is trying to play tricks on us */
+			continue;
+		}
+		fp = fopen (buf, "r");
+		if (fp != NULL) {
+			char buf2[100];
+			if (fgets (buf2, sizeof (buf2), fp) != NULL) {
+				gulong pid;
+				if (sscanf (buf2, "%lu", &pid) == 1 &&
+				    kill (pid, 0) == 0) {
+					fclose (fp);
+					continue;
+				}
+
+			}
+			fclose (fp);
+
+			/* whack the file, it's a stale lock file */
+			unlink (buf);
+		}
+
 		/* if starting as root, we'll be able to overwrite any
 		 * stale sockets or lock files, but a user may not be
 		 * able to */
@@ -356,7 +387,7 @@ gdm_get_free_display (int start, uid_t server_uid)
 			}
 
 			g_snprintf (buf, sizeof (buf),
-				    "/tmp/.%d-lock", i);
+				    "/tmp/.X%d-lock", i);
 			if (stat (buf, &s) == 0 &&
 			    s.st_uid != server_uid) {
 				continue;
@@ -373,53 +404,60 @@ gboolean
 gdm_text_message_dialog (const char *msg)
 {
 	char *dialog; /* do we have dialog?*/
+	char *msg_quoted;
 
-	if (access (EXPANDED_SBINDIR "/gdmopen", X_OK) != 0)
+	if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) != 0)
 		return FALSE;
 
+	msg_quoted = g_shell_quote (msg);
+	
 	dialog = g_find_program_in_path ("dialog");
 	if (dialog == NULL)
 		dialog = g_find_program_in_path ("gdialog");
 	if (dialog == NULL)
 		dialog = g_find_program_in_path ("whiptail");
 	if (dialog != NULL) {
-		char *argv[7];
-
-		argv[0] = EXPANDED_SBINDIR "/gdmopen";
-		argv[1] = dialog;
-		argv[2] = "--msgbox";
-		argv[3] = (char *)msg;
-		argv[4] = "11";
-		argv[5] = "70";
-		argv[6] = NULL;
+		char *argv[6];
+		
+		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
+		argv[1] = "-l";
+		argv[2] = "/bin/sh";
+		argv[3] = "-c";
+		argv[4] = g_strdup_printf ("%s --msgbox %s 11 70",
+					   dialog, msg_quoted);
+		argv[5] = NULL;
 
 		/* make sure gdialog wouldn't get confused */
 		if (gdm_exec_wait (argv, TRUE /* no display */,
 				   TRUE /* de_setuid */) < 0) {
 			g_free (dialog);
+			g_free (msg_quoted);
 			return FALSE;
 		}
 
 		g_free (dialog);
 	} else {
-		char *argv[5];
+		char *argv[6];
 
-		argv[0] = EXPANDED_SBINDIR "/gdmopen";
-		argv[1] = "/bin/sh";
-		argv[2] = "-c";
-		argv[3] = g_strdup_printf
+		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
+		argv[1] = "-l";
+		argv[2] = "/bin/sh";
+		argv[3] = "-c";
+		argv[4] = g_strdup_printf
 			("clear ; "
-			 "echo \"%s\" ; read ; clear",
-			 msg);
-		argv[4] = NULL;
+			 "echo %s ; read ; clear",
+			 msg_quoted);
+		argv[5] = NULL;
 
 		if (gdm_exec_wait (argv, TRUE /* no display */,
 				   TRUE /* de_setuid */) < 0) {
 			g_free (argv[3]);
+			g_free (msg_quoted);
 			return FALSE;
 		}
 		g_free (argv[3]);
 	}
+	g_free (msg_quoted);
 	return TRUE;
 }
 
@@ -427,29 +465,32 @@ gboolean
 gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 {
 	char *dialog; /* do we have dialog?*/
-
-	if (access (EXPANDED_SBINDIR "/gdmopen", X_OK) != 0)
+	char *msg_quoted;
+	
+	if (access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) != 0)
 		return FALSE;
 
 	if (ret != NULL)
 		*ret = FALSE;
 
+	msg_quoted = g_shell_quote (msg);
+	
 	dialog = g_find_program_in_path ("dialog");
 	if (dialog == NULL)
 		dialog = g_find_program_in_path ("gdialog");
 	if (dialog == NULL)
 		dialog = g_find_program_in_path ("whiptail");
 	if (dialog != NULL) {
-		char *argv[7];
+		char *argv[6];
 		int retint;
 
-		argv[0] = EXPANDED_SBINDIR "/gdmopen";
-		argv[1] = dialog;
-		argv[2] = "--yesno";
-		argv[3] = (char *)msg;
-		argv[4] = "11";
-		argv[5] = "70";
-		argv[6] = NULL;
+		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
+		argv[1] = "-l";
+		argv[2] = "/bin/sh";
+		argv[3] = "-c";
+		argv[4] = g_strdup_printf ("%s --yesno %s 11 70",
+					   dialog, msg_quoted);
+		argv[5] = NULL;
 
 		/* will unset DISPLAY and XAUTHORITY if they exist
 		 * so that gdialog (if used) doesn't get confused */
@@ -457,6 +498,7 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 					TRUE /* de_setuid */);
 		if (retint < 0) {
 			g_free (dialog);
+			g_free (msg_quoted);
 			return FALSE;
 		}
 
@@ -464,6 +506,7 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 			*ret = (retint == 0) ? TRUE : FALSE;
 
 		g_free (dialog);
+		g_free (msg_quoted);
 
 		return TRUE;
 	} else {
@@ -471,30 +514,34 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 		int tempfd;
 		FILE *fp;
 		char buf[256];
-		char *argv[5];
+		char *argv[6];
 
 		tempfd = g_mkstemp (tempname);
-		if (tempfd < 0)
+		if (tempfd < 0) {
+			g_free (msg_quoted);
 			return FALSE;
+		}
 
 		close (tempfd);
 
-		argv[0] = EXPANDED_SBINDIR "/gdmopen";
-		argv[1] = "/bin/sh";
-		argv[2] = "-c";
-		argv[3] = g_strdup_printf
+		argv[0] = EXPANDED_LIBEXECDIR "/gdmopen";
+		argv[1] = "-l";
+		argv[2] = "/bin/sh";
+		argv[3] = "-c";
+		argv[4] = g_strdup_printf
 			("clear ; "
-			 "echo \"%s\" ; echo ; echo \"%s\" ; "
+			 "echo %s ; echo ; echo \"%s\" ; "
 			 "read RETURN ; echo $RETURN > %s ; clear'",
-			 msg,
+			 msg_quoted,
 			 /* Translators, don't translate the 'y' and 'n' */
 			 _("y = Yes or n = No? >"),
 			 tempname);
-		argv[4] = NULL;
+		argv[5] = NULL;
 
 		if (gdm_exec_wait (argv, TRUE /* no display */,
 				   TRUE /* de_setuid */) < 0) {
 			g_free (argv[3]);
+			g_free (msg_quoted);
 			return FALSE;
 		}
 		g_free (argv[3]);
@@ -507,12 +554,14 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 					*ret = TRUE;
 				fclose (fp);
 			} else {
+				g_free (msg_quoted);
 				return FALSE;
 			}
 		}
 
 		unlink (tempname);
 
+		g_free (msg_quoted);
 		return TRUE;
 	}
 }
@@ -533,7 +582,7 @@ gdm_exec_wait (char * const *argv, gboolean no_display,
 	if (pid == 0) {
 		closelog ();
 
-		gdm_close_all_descriptors (0 /* from */, -1 /* except */);
+		gdm_close_all_descriptors (0 /* from */, -1 /* except */, -1 /* except2 */);
 
 		/* No error checking here - if it's messed the best response
 		 * is to ignore & try to continue */
@@ -548,8 +597,8 @@ gdm_exec_wait (char * const *argv, gboolean no_display,
 		openlog ("gdm", LOG_PID, LOG_DAEMON);
 
 		if (no_display) {
-			gnome_unsetenv ("DISPLAY");
-			gnome_unsetenv ("XAUTHORITY");
+			ve_unsetenv ("DISPLAY");
+			ve_unsetenv ("XAUTHORITY");
 		}
 		
 		execv (argv[0], argv);
@@ -655,14 +704,20 @@ pid_t
 gdm_fork_extra (void)
 {
 	pid_t pid;
-	gdm_sigchld_block_push ();
 
+	gdm_sigchld_block_push ();
 	gdm_sigterm_block_push ();
+
 	pid = extra_process = fork ();
 	if (pid < 0)
 		extra_process = 0;
-	gdm_sigterm_block_pop ();
+	else if (pid == 0)
+		/* unset signals here, and yet again
+		   later as the block_pop will whack
+		   our signal mask*/
+		gdm_unset_signals ();
 
+	gdm_sigterm_block_pop ();
 	gdm_sigchld_block_pop ();
 
 	if (pid == 0) {
@@ -674,6 +729,12 @@ gdm_fork_extra (void)
 		 * kill -(extra_process) to kill extra process and all it's
 		 * possible children */
 		setsid ();
+
+		/* Harmless in children, but in case we'd run
+		   extra processes from main daemon would fix
+		   problems ... */
+		if (GdmXdmcp)
+			gdm_xdmcp_close ();
 	}
 
 	return pid;
@@ -818,7 +879,7 @@ gdm_peek_local_address_list (void)
 	if (gethostname (hostbuf, BUFSIZ-1) != 0) {
 		gdm_debug ("%s: Could not get server hostname: %s!",
 			   "gdm_peek_local_address_list",
-			   g_strerror (errno));
+			   strerror (errno));
 		addr = g_new0 (struct in_addr, 1);
 		addr->s_addr = INADDR_LOOPBACK;
 		return g_list_append (NULL, addr);
@@ -1030,38 +1091,16 @@ gdm_fdgets (int fd)
 }
 
 void
-gdm_close_all_descriptors (int from, int except)
+gdm_close_all_descriptors (int from, int except, int except2)
 {
 	int i;
 	int max = sysconf (_SC_OPEN_MAX);
 	for (i = from; i < max; i++) {
-		if (i != except)
+		if (i != except && i != except2)
 			close(i);
 	}
 }
 
-gboolean
-gdm_is_a_no_password_user (const char *user)
-{
-	char **vector = NULL;
-	int i;
-
-	if (ve_string_empty (GdmLocalNoPasswordUsers) ||
-	    ve_string_empty (user) ||
-	    strcmp (user, "root") == 0)
-		return FALSE;
-
-	vector = g_strsplit (GdmLocalNoPasswordUsers, ",", -1);
-	for (i = 0; vector[i] != NULL; i++) {
-		if (strcmp (vector[i], user) == 0) {
-			g_strfreev (vector);
-			return TRUE;
-		}
-	}
-	g_strfreev (vector);
-	return FALSE;
-}
-
 int
 gdm_open_dev_null (mode_t mode)
 {
@@ -1108,7 +1147,6 @@ gdm_locale_to_utf8 (const char *text)
 {
 	GIConv cd;
 	char *out;
-	GError *error = NULL;
 
 	if (gdm_charset == NULL) {
 		return g_strdup (text);
@@ -1119,7 +1157,7 @@ gdm_locale_to_utf8 (const char *text)
 		return ascify (text);
 	}
 
-	out = g_convert_with_iconv (text, -1, cd, NULL, NULL, &error);
+	out = g_convert_with_iconv (text, -1, cd, NULL, NULL, NULL /* error */);
 	g_iconv_close (cd);
 	if (out == NULL) {
 		return ascify (text);
@@ -1142,13 +1180,337 @@ gdm_locale_from_utf8 (const char *text)
 		return ascify (text);
 	}
 
-	out = g_convert_with_iconv (text, -1, cd, NULL, NULL, NULL);
+	out = g_convert_with_iconv (text, -1, cd, NULL, NULL, NULL /* error */);
 	g_iconv_close (cd);
 	if (out == NULL)
 		return ascify (text);
 	return out;
 }
 
+static GdmHostent *
+fillout_hostent (struct hostent *he_, struct in_addr *ia, const char *name)
+{
+	GdmHostent *he;
 
+	he = g_new0 (GdmHostent, 1);
+
+	he->addrs = NULL;
+	he->addr_count = 0;
+
+	/* Sometimes if we can't look things up, we could end
+	   up with a dot in the name field which would screw
+	   us up.  Weird but apparently possible */
+	if (he_ != NULL &&
+	    he_->h_name != NULL &&
+	    he_->h_name[0] != '\0' &&
+	    strcmp (he_->h_name, ".") != 0) {
+		he->hostname = g_strdup (he_->h_name);
+		he->not_found = FALSE;
+	} else {
+		he->not_found = TRUE;
+		if (name != NULL)
+			he->hostname = g_strdup (name);
+		else /* either ia or name is set */
+			he->hostname = g_strdup (inet_ntoa (*ia));
+	}
+
+	if (he_ != NULL && he_->h_addrtype == AF_INET) {
+		int i;
+		for (i = 0; ; i++) {
+			struct in_addr *ia_ = (struct in_addr *) (he_->h_addr_list[i]);
+			if (ia_ == NULL)
+				break;
+		}
+		he->addrs = g_new0 (struct in_addr, i);
+		he->addr_count = i;
+		for (i = 0; ; i++) {
+			struct in_addr *ia_ = (struct in_addr *) he_->h_addr_list[i];
+			if (ia_ == NULL)
+				break;
+			(he->addrs)[i] = *ia_;
+		}
+	}
+	return he;
+}
+
+static gboolean do_jumpback = FALSE;
+static Jmp_buf signal_jumpback;
+static struct sigaction oldterm, oldint, oldhup;
+
+static void
+jumpback_sighandler (int signal)
+{
+	if (signal == SIGINT)
+		oldint.sa_handler (signal);
+	else if (signal == SIGTERM)
+		oldint.sa_handler (signal);
+	else if (signal == SIGHUP)
+		oldint.sa_handler (signal);
+	/* no others should be set up */
+	
+	if (do_jumpback) {
+		do_jumpback = FALSE;
+		Longjmp (signal_jumpback, 1);
+	}
+}
+
+#define SETUP_INTERRUPTS_FOR_TERM_DECLS \
+    struct sigaction term;
+
+#define SETUP_INTERRUPTS_FOR_TERM_SETUP \
+    do_jumpback = FALSE;						\
+    									\
+    term.sa_handler = jumpback_sighandler;				\
+    term.sa_flags = SA_RESTART;						\
+    sigemptyset (&term.sa_mask);					\
+									\
+    if (sigaction (SIGTERM, &term, &oldterm) < 0) 			\
+	gdm_fail (_("%s: Error setting up TERM signal handler"),	\
+		  "SETUP_INTERRUPTS_FOR_TERM");				\
+									\
+    if (sigaction (SIGINT, &term, &oldint) < 0)				\
+	gdm_fail (_("%s: Error setting up INT signal handler"),		\
+		  "SETUP_INTERRUPTS_FOR_TERM");				\
+									\
+    if (sigaction (SIGHUP, &term, &oldhup) < 0) 			\
+	gdm_fail (_("%s: Error setting up HUP signal handler"),		\
+		  "SETUP_INTERRUPTS_FOR_TERM");
+
+#define SETUP_INTERRUPTS_FOR_TERM_TEARDOWN \
+    do_jumpback = FALSE;						\
+									\
+    if (sigaction (SIGTERM, &oldterm, NULL) < 0) 			\
+	gdm_fail (_("%s: Error setting up TERM signal handler"),	\
+		  "SETUP_INTERRUPTS_FOR_TERM");				\
+									\
+    if (sigaction (SIGINT, &oldint, NULL) < 0) 				\
+	gdm_fail (_("%s: Error setting up INT signal handler"),		\
+		  "SETUP_INTERRUPTS_FOR_TERM");				\
+									\
+    if (sigaction (SIGHUP, &oldhup, NULL) < 0) 				\
+	gdm_fail (_("%s: Error setting up HUP signal handler"),		\
+		  "SETUP_INTERRUPTS_FOR_TERM");
+
+GdmHostent *
+gdm_gethostbyname (const char *name)
+{
+	struct hostent *he_;
+	SETUP_INTERRUPTS_FOR_TERM_DECLS
+
+	/* the cached address */
+	static GdmHostent *he = NULL;
+	static time_t last_time = 0;
+	static char *cached_hostname = NULL;
+
+	if (cached_hostname != NULL &&
+	    strcmp (cached_hostname, name) == 0) {
+		/* don't check more then every 60 seconds */
+		if (last_time + 60 > time (NULL))
+			return gdm_hostent_copy (he);
+	}
+
+	SETUP_INTERRUPTS_FOR_TERM_SETUP
+
+	if (Setjmp (signal_jumpback) == 0) {
+		do_jumpback = TRUE;
+		/* Find client hostname */
+		he_ = gethostbyname (name);
+		do_jumpback = FALSE;
+	} else {
+		/* here we got interrupted */
+		he_ = NULL;
+	}
+
+	SETUP_INTERRUPTS_FOR_TERM_TEARDOWN
+
+	g_free (cached_hostname);
+	cached_hostname = g_strdup (name);
+
+	gdm_hostent_free (he);
+	he = fillout_hostent (he_, NULL, name);
+
+	last_time = time (NULL);
+	return gdm_hostent_copy (he);
+}
+
+GdmHostent *
+gdm_gethostbyaddr (struct in_addr *ia)
+{
+	struct hostent *he_;
+	SETUP_INTERRUPTS_FOR_TERM_DECLS
+
+	/* the cached address */
+	static GdmHostent *he = NULL;
+	static time_t last_time = 0;
+	static struct in_addr cached_addr;
+
+	if (last_time != 0 &&
+	    memcmp (&cached_addr, ia, sizeof (struct in_addr)) == 0) {
+		/* don't check more then every 60 seconds */
+		if (last_time + 60 > time (NULL))
+			return gdm_hostent_copy (he);
+	}
+
+	SETUP_INTERRUPTS_FOR_TERM_SETUP
+
+	if (Setjmp (signal_jumpback) == 0) {
+		do_jumpback = TRUE;
+		/* Find client hostname */
+		he_ = gethostbyaddr ((gchar *) ia, sizeof (struct in_addr), AF_INET);
+		do_jumpback = FALSE;
+	} else {
+		/* here we got interrupted */
+		he_ = NULL;
+	}
+
+	SETUP_INTERRUPTS_FOR_TERM_TEARDOWN
+
+	cached_addr = *ia;
+
+	gdm_hostent_free (he);
+	he = fillout_hostent (he_, ia, NULL);
+
+	last_time = time (NULL);
+	return gdm_hostent_copy (he);
+}
+
+GdmHostent *
+gdm_hostent_copy (GdmHostent *he)
+{
+	GdmHostent *cpy;
+
+	if (he == NULL)
+		return NULL;
+
+	cpy = g_new0 (GdmHostent, 1);
+	cpy->not_found = he->not_found;
+	cpy->hostname = g_strdup (he->hostname);
+	if (he->addr_count == 0) {
+		cpy->addr_count = 0;
+		cpy->addrs = NULL;
+	} else {
+		cpy->addr_count = he->addr_count;
+		cpy->addrs = g_new0 (struct in_addr, he->addr_count);
+		memcpy (cpy->addrs, he->addrs, sizeof (struct in_addr) * he->addr_count);
+	}
+	return cpy;
+}
+
+void
+gdm_hostent_free (GdmHostent *he)
+{
+	if (he == NULL)
+		return;
+	g_free (he->hostname);
+	he->hostname = NULL;
+
+	g_free (he->addrs);
+	he->addrs = NULL;
+	he->addr_count = 0;
+
+	g_free (he);
+}
+
+/* like fopen with "w" */
+FILE *
+gdm_safe_fopen_w (const char *file)
+{
+	int fd;
+	unlink (file);
+	fd = open (file, O_EXCL|O_CREAT|O_TRUNC|O_WRONLY, 0644);
+	if (fd < 0)
+		return NULL;
+	return fdopen (fd, "w");
+}
+
+/* like fopen with "a+" */
+FILE *
+gdm_safe_fopen_ap (const char *file)
+{
+	int fd;
+
+	if (access (file, F_OK) == 0) {
+#ifdef O_NOFOLLOW
+		fd = open (file, O_APPEND|O_RDWR|O_NOFOLLOW);
+#else
+		fd = open (file, O_APPEND|O_RDWR);
+#endif
+	} else {
+		/* doesn't exist, open with O_EXCL */
+		fd = open (file, O_EXCL|O_CREAT|O_RDWR, 0644);
+	}
+	if (fd < 0)
+		return NULL;
+	return fdopen (fd, "a+");
+}
+
+void
+gdm_reset_limits (void)
+{
+	struct rlimit unlim = { RLIM_INFINITY, RLIM_INFINITY };
+
+	/* Note: I don't really know which ones are really very standard
+	   and which ones are not, so I just test for them all one by one */
+
+#ifdef RLIMIT_CPU
+	setrlimit (RLIMIT_CPU, &unlim);
+#endif
+#ifdef RLIMIT_DATA
+	setrlimit (RLIMIT_DATA, &unlim);
+#endif
+#ifdef RLIMIT_FSIZE
+	setrlimit (RLIMIT_FSIZE, &unlim);
+#endif
+#ifdef RLIMIT_LOCKS
+	setrlimit (RLIMIT_LOCKS, &unlim);
+#endif
+#ifdef RLIMIT_MEMLOCK
+	setrlimit (RLIMIT_MEMLOCK, &unlim);
+#endif
+#ifdef RLIMIT_NOFILE
+	setrlimit (RLIMIT_NOFILE, &unlim);
+#endif
+#ifdef RLIMIT_OFILE
+	setrlimit (RLIMIT_OFILE, &unlim);
+#endif
+#ifdef RLIMIT_NPROC
+	setrlimit (RLIMIT_NPROC, &unlim);
+#endif
+#ifdef RLIMIT_RSS
+	setrlimit (RLIMIT_RSS, &unlim);
+#endif
+#ifdef RLIMIT_STACK
+	setrlimit (RLIMIT_STACK, &unlim);
+#endif
+#ifdef RLIMIT_CORE
+	setrlimit (RLIMIT_CORE, &unlim);
+#endif
+#ifdef RLIMIT_AS
+	setrlimit (RLIMIT_AS, &unlim);
+#endif
+#ifdef RLIMIT_VMEM
+	setrlimit (RLIMIT_VMEM, &unlim);
+#endif
+#ifdef RLIMIT_PTHREAD
+	setrlimit (RLIMIT_PTHREAD, &unlim);
+#endif
+}
+
+const char *
+gdm_root_user (void)
+{
+	static char *root_user = NULL;
+	struct passwd *pwent;
+
+	if (root_user != NULL)
+		return root_user;
+
+	pwent = getpwuid (0);
+	if (pwent == NULL) /* huh? */
+		root_user = g_strdup ("root");
+	else
+		root_user = g_strdup (pwent->pw_name);
+	return root_user;
+}
 
 /* EOF */
