@@ -75,6 +75,34 @@ static GdmDisplay *d = NULL;
 static gboolean server_signal_notified = FALSE;
 static int server_signal_pipe[2];
 
+void
+gdm_server_whack_lockfile (GdmDisplay *disp)
+{
+	    char buf[256];
+
+	    /* X seems to be sometimes broken with its lock files and
+	       doesn't seem to remove them always, and if you manage
+	       to get into the weird state where the old pid now
+	       corresponds to some new pid, X will just die with
+	       a stupid error. */
+
+	    /* Yes there could be a race here if another X server starts
+	       at this exact instant.  Oh well such is life.  Very unlikely
+	       to happen though as we should really be the only ones
+	       trying to start X servers, and we aren't starting an
+	       X server on this display yet. */
+
+	    /* if lock file exists and it is our process, whack it! */
+	    g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", disp->dispnum);
+	    unlink (buf);
+
+	    /* whack the unix socket as well */
+	    g_snprintf (buf, sizeof (buf),
+			"/tmp/.X11-unix/X%d", disp->dispnum);
+	    unlink (buf);
+}
+
+
 /* Wipe cookie files */
 void
 gdm_server_wipe_cookies (GdmDisplay *disp)
@@ -89,6 +117,8 @@ gdm_server_wipe_cookies (GdmDisplay *disp)
 	disp->authfile_gdm = NULL;
 }
 
+static Jmp_buf reinitjmp;
+
 /* ignore handlers */
 static int
 ignore_xerror_handler (Display *disp, XErrorEvent *evt)
@@ -97,9 +127,9 @@ ignore_xerror_handler (Display *disp, XErrorEvent *evt)
 }
 
 static int
-exit_xioerror_handler (Display *disp)
+jumpback_xioerror_handler (Display *disp)
 {
-	_exit (1);
+	Longjmp (reinitjmp, 1);
 }
 
 /**
@@ -113,86 +143,69 @@ exit_xioerror_handler (Display *disp)
 void
 gdm_server_reinit (GdmDisplay *disp)
 {
-	pid_t pid;
+	int (*old_xerror_handler)(Display *, XErrorEvent *) = NULL;
+	int (*old_xioerror_handler)(Display *) = NULL;
 
 	if (disp == NULL)
 		return;
 
-	/* Kill our connection if one existed */
-	if (disp->dsp != NULL) {
-		XCloseDisplay (disp->dsp);
-		disp->dsp = NULL;
-	}
-
-	if (disp->servpid <= 0)
+	if (disp->servpid <= 0) {
+		/* Kill our connection if one existed, likely to result
+		 * in some bizzaro error right now */
+		if (disp->dsp != NULL) {
+			XCloseDisplay (disp->dsp);
+			disp->dsp = NULL;
+		}
 		return;
+	}
 
 	gdm_debug ("gdm_server_reinit: Server for %s is about to be reinitialized!", disp->name);
 
-	/* This hack below is beautiful BTW */
+	/* Long live Setjmp, DIE DIE DIE XSetIOErrorHandler */
 
-	gdm_sigchld_block_push ();
-
-	pid = gdm_fork_extra ();
-	if (pid < 0) {
-		/* So much work just because we can't fork */
-		if (disp->servpid > 1)
-			kill (disp->servpid, SIGHUP);
-
-		gdm_sigchld_block_push ();
-
-		/* HACK! the Xserver can't really tell us when it got the hup signal,
-		 * so we are really stuck just going to sleep for a bit hoping that
-		 * the kernel will tell the X server and that will run, else we will
-		 * get whacked ourselves after we open the connection and we'll think
-		 * it's an X screwup, which is really OK to happen and will just
-		 * restart the Xserver, it's just more nasty.  Oh how fun */
-		sleep (1);
-	} else if (pid == 0) {
-		Display *dsp;
-
-		closelog ();
-
-		signal (SIGCHLD, SIG_IGN);
-		signal (SIGTERM, SIG_DFL);
-		signal (SIGINT, SIG_DFL);
-		signal (SIGHUP, SIG_DFL);
-
-		/* close things */
-		gdm_close_all_descriptors (0 /* from */, -1 /* except */);
-
-		gdm_open_dev_null (O_RDONLY); /* open stdin - fd 0 */
-		gdm_open_dev_null (O_RDWR); /* open stdout - fd 1 */
-		gdm_open_dev_null (O_RDWR); /* open stderr - fd 2 */
-
-		XSetErrorHandler (ignore_xerror_handler);
-		XSetIOErrorHandler (exit_xioerror_handler);
-	       
-		gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
-		dsp = XOpenDisplay (d->name);
+	if (disp->dsp == NULL) {
+		gdm_error ("Reiniting server, but don't have a display connection, strange");
 
 		/* Now whack the server with a SIGHUP */
+		gdm_sigchld_block_push ();
 		if (disp->servpid > 1)
 			kill (disp->servpid, SIGHUP);
+		gdm_sigchld_block_pop ();
+		/* a hack we have no way of knowing when the server died */
+		sleep (1);
+	}
 
-		if (dsp == NULL) {
-			/* HACK, see note above */
-			sleep (1);
-			_exit (0);
+	if (Setjmp (reinitjmp) == 0)  {
+		/* come here and we'll whack the server and wait to get
+		   an xio error */
+		old_xerror_handler = XSetErrorHandler (ignore_xerror_handler);
+		old_xioerror_handler = XSetIOErrorHandler (jumpback_xioerror_handler);
+
+		/* Now whack the server with a SIGHUP */
+		gdm_sigchld_block_push ();
+		if (disp->servpid > 1) {
+			kill (disp->servpid, SIGHUP);
 		} else {
-			/* Wait for an xioerror */
-			for (;;) {
-				XEvent event;
-				XNextEvent (dsp, &event);
+			gdm_sigchld_block_pop ();
+			/* the server is dead, weird */
+			if (disp->dsp != NULL) {
+				XCloseDisplay (disp->dsp);
+				disp->dsp = NULL;
 			}
+			Longjmp (reinitjmp, 1);
 		}
-		/* anality */
-		_exit (0);
-	} else if (pid > 0) {
 		gdm_sigchld_block_pop ();
 
-		gdm_wait_for_extra (NULL);
+		/* Wait for an xioerror */
+		for (;;) {
+			XEvent event;
+			XNextEvent (disp->dsp, &event);
+		}
 	}
+	/* no more display */
+	disp->dsp = NULL;
+	XSetErrorHandler (old_xerror_handler);
+	XSetIOErrorHandler (old_xioerror_handler);
 }
 
 /**
@@ -238,6 +251,7 @@ gdm_server_stop (GdmDisplay *disp)
 		    ve_waitpid_no_signal (servpid, 0, 0);
 	    gdm_sigchld_block_pop ();
 
+	    gdm_server_whack_lockfile (disp);
 
 	    gdm_debug ("gdm_server_stop: Server pid %d dead", (int)servpid);
     }
@@ -281,6 +295,7 @@ busy_ask_user (GdmDisplay *disp)
     }
 }
 
+/* Checks only output, no XFree86 v4 logfile */
 static gboolean
 display_xnest_no_connect (GdmDisplay *disp)
 {
@@ -324,9 +339,7 @@ display_busy (GdmDisplay *disp)
 		return FALSE;
 
 	while (fgets (buf, sizeof (buf), fp) != NULL) {
-		/* Note: this is probably XFree86 specific, and perhaps even
-		 * version 3 specific (I don't have xfree v4 to test this),
-		 * of course additions are welcome to make this more robust */
+		/* Note: this is probably XFree86 specific */
 		if (strstr (buf, "Server is already active for display")
 		    == buf) {
 			gdm_error (_("Display %s is busy. There is another "
@@ -341,12 +354,36 @@ display_busy (GdmDisplay *disp)
 	return FALSE;
 }
 
+/* if we find 'Log file: "foo"' switch fp to foo and
+   return TRUE */
+/* Note: assumes buf is of size 256 and is writable */ 
+static gboolean
+open_another_logfile (char buf[256], FILE **fp)
+{
+	if (strncmp (&buf[5], "Log file: \"", strlen ("Log file: \"")) == 0) {
+		FILE *ffp;
+		char *fname = &buf[5+strlen ("Log file: \"")];
+		char *p = strchr (fname, '"');
+		if (p == NULL)
+			return FALSE;
+		*p = '\0';
+		ffp = fopen (fname, "r");
+		if (ffp == NULL)
+			return FALSE;
+		fclose (*fp);
+		*fp = ffp;
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static int
 display_vt (GdmDisplay *disp)
 {
 	char *logname = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
 	FILE *fp;
 	char buf[256];
+	gboolean switched = FALSE;
 
 	fp = fopen (logname, "r");
 	g_free (logname);
@@ -356,10 +393,19 @@ display_vt (GdmDisplay *disp)
 
 	while (fgets (buf, sizeof (buf), fp) != NULL) {
 		int vt;
-		/* Note: this is probably XFree86 specific, and perhaps even
-		 * version 3 specific (I don't have xfree v4 to test this),
-		 * of course additions are welcome to make this more robust */
-		if (sscanf (buf, "(using VT number %d)", &vt) == 1) {
+		char *p;
+
+		if ( ! switched &&
+		     /* this is XFree v4 specific */
+		    open_another_logfile (buf, &fp)) {
+			switched = TRUE;
+			continue;
+		} 
+		/* Note: this is probably XFree86 specific (works with
+		 * both v3 and v4 though */
+		p = strstr (buf, "using VT number ");
+		if (p != NULL &&
+		    sscanf (p, "using VT number %d", &vt) == 1) {
 			fclose (fp);
 			return vt;
 		}
@@ -453,7 +499,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     if ( ! gdm_auth_secure_display (d)) 
 	    return FALSE;
     gdm_slave_send_string (GDM_SOP_COOKIE, d->cookie);
-    gnome_setenv ("DISPLAY", d->name, TRUE);
+    ve_setenv ("DISPLAY", d->name, TRUE);
 
     if (pipe (server_signal_pipe) != 0) {
 	    gdm_error (_("%s: Error opening a pipe: %s"),
@@ -556,7 +602,11 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 		    /* In case we got a SIGCHLD */
 		    check_child_status ();
 
-		    gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
+		    /* just in case it's set */
+		    ve_unsetenv ("XAUTHORITY");
+
+		    gdm_auth_set_local_auth (d);
+
 		    for (i = 0;
 			 d->dsp == NULL &&
 			 d->servstat == SERVER_STARTED &&
@@ -571,7 +621,6 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 			    /* In case we got a SIGCHLD */
 			    check_child_status ();
 		    }
-		    gnome_unsetenv ("XAUTHORITY");
 		    if (d->dsp == NULL &&
 			/* Note: we could have still gotten a SIGCHLD */
 			d->servstat == SERVER_STARTED) {
@@ -662,6 +711,8 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 		kill (pid, SIGTERM) == 0)
 		    ve_waitpid_no_signal (pid, NULL, 0);
 	    gdm_sigchld_block_pop ();
+	    
+	    gdm_server_whack_lockfile (disp);
     }
 
     /* We will rebake cookies anyway, so wipe these */
@@ -747,6 +798,27 @@ gdm_server_checklog (GdmDisplay *disp)
 	}
 }
 
+/* somewhat safer rename (safer if the log dir is unsafe), may in fact
+   lose the file though, it guarantees that a is gone, but not that
+   b exists */
+static void
+safer_rename (const char *a, const char *b)
+{
+	errno = 0;
+	if (link (a, b) < 0) {
+		if (errno == EEXIST) {
+			unlink (a);
+			return;
+		} 
+		unlink (b);
+		/* likely this system doesn't support hard links */
+		rename (a, b);
+		unlink (a);
+		return;
+	}
+	unlink (a);
+}
+
 static void
 rotate_logs (const char *dname)
 {
@@ -759,10 +831,10 @@ rotate_logs (const char *dname)
 
 	/* Rotate the logs (keep 4 last) */
 	unlink (fname4);
-	rename (fname3, fname4);
-	rename (fname2, fname3);
-	rename (fname1, fname2);
-	rename (fname, fname1);
+	safer_rename (fname3, fname4);
+	safer_rename (fname2, fname3);
+	safer_rename (fname1, fname2);
+	safer_rename (fname, fname1);
 
 	g_free (fname4);
 	g_free (fname3);
@@ -895,6 +967,7 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
     struct sigaction ign_signal, dfl_signal;
     sigset_t mask;
     gchar **argv = NULL;
+    char *logfile;
     int logfd;
     char *command;
     pid_t pid;
@@ -929,6 +1002,8 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 
     gdm_sigterm_block_push ();
     pid = d->servpid = fork ();
+    if (pid == 0)
+	    gdm_unset_signals ();
     gdm_sigterm_block_pop ();
     gdm_sigchld_block_pop ();
     
@@ -944,7 +1019,7 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 	closelog ();
 
 	/* close things */
-	gdm_close_all_descriptors (0 /* from */, -1 /* except */);
+	gdm_close_all_descriptors (0 /* from */, -1 /* except */, -1 /* except2 */);
 
 	/* No error checking here - if it's messed the best response
          * is to ignore & try to continue */
@@ -958,8 +1033,9 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 	rotate_logs (d->name);
 
         /* Log all output from spawned programs to a file */
-	logfd = open (g_strconcat (GdmLogDir, "/", d->name, ".log", NULL),
-		      O_CREAT|O_TRUNC|O_WRONLY, 0644);
+	logfile = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
+	unlink (logfile);
+	logfd = open (logfile, O_CREAT|O_TRUNC|O_WRONLY|O_EXCL, 0644);
 
 	if (logfd != -1) {
 		dup2 (logfd, 1);
@@ -1014,11 +1090,11 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 
 	if (d->type == TYPE_FLEXI_XNEST) {
 		char *font_path = NULL;
-		gnome_setenv ("DISPLAY", d->xnest_disp, TRUE);
+		ve_setenv ("DISPLAY", d->xnest_disp, TRUE);
 		if (d->xnest_auth_file != NULL)
-			gnome_setenv ("XAUTHORITY", d->xnest_auth_file, TRUE);
+			ve_setenv ("XAUTHORITY", d->xnest_auth_file, TRUE);
 		else
-			gnome_unsetenv ("XAUTHORITY");
+			ve_unsetenv ("XAUTHORITY");
 
 		/* Add -fp with the current font path, but only if not
 		 * already among the arguments */
@@ -1058,11 +1134,11 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 		}
 		if (pwent->pw_dir != NULL &&
 		    g_file_test (pwent->pw_dir, G_FILE_TEST_EXISTS))
-			gnome_setenv ("HOME", pwent->pw_dir, TRUE);
+			ve_setenv ("HOME", pwent->pw_dir, TRUE);
 		else
-			gnome_setenv ("HOME", "/", TRUE); /* Hack */
-		gnome_setenv ("SHELL", pwent->pw_shell, TRUE);
-		gnome_unsetenv ("MAIL");
+			ve_setenv ("HOME", "/", TRUE); /* Hack */
+		ve_setenv ("SHELL", pwent->pw_shell, TRUE);
+		ve_unsetenv ("MAIL");
 
 		if (setgid (pwent->pw_gid) < 0)  {
 			gdm_error (_("%s: Couldn't set groupid to %d"), 
