@@ -24,6 +24,9 @@
 /* Wipe cookie files */
 void		gdm_server_wipe_cookies	(GdmDisplay *disp);
 
+/* Wipe X server's butt */
+void		gdm_server_whack_lockfile (GdmDisplay *disp);
+
 gboolean	gdm_server_start	(GdmDisplay *d,
 					 gboolean treat_as_flexi,
 					 int min_flexi_disp,
