@@ -97,7 +97,6 @@
 #include "choose.h"
 #include "xdmcp.h"
 
-gint pending = 0;
 int gdm_xdmcpfd = -1;
 
 #ifdef HAVE_LIBXDMCP
@@ -107,14 +106,15 @@ gint allow_severity = LOG_INFO;
 gint deny_severity = LOG_WARNING;
 
 static guint xdmcp_source = 0;
-static gint globsessid;
+static CARD32 globsessid;
 static gchar *sysid;
 static ARRAY8 servhost;
 static XdmcpBuffer buf;
 static gboolean initted = FALSE;
 
 extern GSList *displays;
-extern gint sessions;
+extern gint xdmcp_pending;
+extern gint xdmcp_sessions;
 extern gchar *GdmLogDir;
 extern gchar *GdmServAuthDir;
 
@@ -153,7 +153,9 @@ static void gdm_xdmcp_whack_queued_managed_forwards (struct sockaddr_in *clnt_sa
 						     struct in_addr *origin);
 static void gdm_xdmcp_send_willing (struct sockaddr_in *clnt_sa);
 static void gdm_xdmcp_send_unwilling (struct sockaddr_in *clnt_sa, gint type);
-static void gdm_xdmcp_send_accept (const char *hostname, struct sockaddr_in *clnt_sa, gint displaynum);
+static void gdm_xdmcp_send_accept (GdmHostent *he /* eaten and freed */,
+				   struct sockaddr_in *clnt_sa,
+				   int displaynum);
 static void gdm_xdmcp_send_decline (struct sockaddr_in *clnt_sa, const char *reason);
 static void gdm_xdmcp_send_refuse (struct sockaddr_in *clnt_sa, CARD32 sessid);
 static void gdm_xdmcp_send_failed (struct sockaddr_in *clnt_sa, CARD32 sessid);
@@ -163,7 +165,9 @@ static void gdm_xdmcp_send_managed_forward (struct sockaddr_in *clnt_sa,
 static void gdm_xdmcp_send_got_managed_forward (struct sockaddr_in *clnt_sa,
 						struct in_addr *origin);
 static gboolean gdm_xdmcp_host_allow (struct sockaddr_in *cnlt_sa);
-static GdmDisplay *gdm_xdmcp_display_alloc (struct in_addr *addr, const char *hostname, gint);
+static GdmDisplay *gdm_xdmcp_display_alloc (struct in_addr *addr,
+					    GdmHostent *he /* eaten and freed */,
+					    int displaynum);
 static GdmDisplay *gdm_xdmcp_display_lookup (CARD32 sessid);
 static void gdm_xdmcp_display_dispose_check (const gchar *name);
 static void gdm_xdmcp_displays_check (void);
@@ -241,7 +245,8 @@ gdm_xdmcp_init (void)
     /* Fetch and store local hostname in XDMCP friendly format */
     hostbuf[1023] = '\0';
     if (gethostname (hostbuf, 1023) != 0) {
-	gdm_error (_("gdm_xdmcp_init: Could not get server hostname: %s!"), strerror (errno));
+	gdm_error (_("%s: Could not get server hostname: %s!"), 
+		   "gdm_xdmcp_init", strerror (errno));
 	strcmp (hostbuf, "localhost.localdomain");
     }
 
@@ -261,7 +266,7 @@ gdm_xdmcp_init (void)
     gdm_xdmcpfd = socket (AF_INET, SOCK_DGRAM, 0); /* UDP */
     
     if (gdm_xdmcpfd < 0) {
-	gdm_error (_("gdm_xdmcp_init: Could not create socket!"));
+	gdm_error (_("%s: Could not create socket!"), "gdm_xdmcp_init");
 	GdmXdmcp = FALSE;
 	return FALSE;
     }
@@ -271,7 +276,7 @@ gdm_xdmcp_init (void)
     serv_sa.sin_addr.s_addr = htonl (INADDR_ANY);
     
     if (bind (gdm_xdmcpfd, (struct sockaddr*) &serv_sa, sizeof (serv_sa)) == -1) {
-	gdm_error (_("gdm_xdmcp_init: Could not bind to XDMCP socket!"));
+	gdm_error (_("%s: Could not bind to XDMCP socket!"), "gdm_xdmcp_init");
 	gdm_xdmcp_close ();
 	GdmXdmcp = FALSE;
 	return FALSE;
@@ -330,18 +335,21 @@ gdm_xdmcp_decode_packet (GIOChannel *source, GIOCondition cond, gpointer data)
     };
     
     if (!XdmcpFill (gdm_xdmcpfd, &buf, (XdmcpNetaddr)&clnt_sa, &sa_len)) {
-	gdm_error (_("gdm_xdmcp_decode: Could not create XDMCP buffer!"));
+	gdm_error (_("%s: Could not create XDMCP buffer!"),
+		   "gdm_xdmcp_decode_packet");
 	return TRUE;
     }
     
     if (!XdmcpReadHeader (&buf, &header)) {
-	gdm_error (_("gdm_xdmcp_decode: Could not read XDMCP header!"));
+	gdm_error (_("%s: Could not read XDMCP header!"),
+		   "gdm_xdmcp_decode_packet");
 	return TRUE;
     }
     
     if (header.version != XDM_PROTOCOL_VERSION &&
 	header.version != GDM_XDMCP_PROTOCOL_VERSION) {
-	gdm_error (_("gdm_xdmcp_decode: Incorrect XDMCP version!"));
+	gdm_error (_("%s: Incorrect XDMCP version!"),
+		   "gdm_xdmcp_decode_packet");
 	return TRUE;
     }
 
@@ -394,7 +402,8 @@ gdm_xdmcp_decode_packet (GIOChannel *source, GIOCondition cond, gpointer data)
 	break;
 	
     default:
-	gdm_error (_("gdm_xdmcp_decode_packet: Unknown opcode from host %s"),
+	gdm_error (_("%s: Unknown opcode from host %s"),
+		   "gdm_xdmcp_decode_packet",
 		   inet_ntoa (clnt_sa.sin_addr));
 	break;
     }
@@ -413,7 +422,8 @@ gdm_xdmcp_handle_query (struct sockaddr_in *clnt_sa, gint len, gint type)
     
     /* Extract array of authentication names from Xdmcp packet */
     if (! XdmcpReadARRAYofARRAY8 (&buf, &clnt_authlist)) {
-	gdm_error (_("gdm_xdmcp_handle_query: Could not extract authlist from packet")); 
+	gdm_error (_("%s: Could not extract authlist from packet"),
+		   "gdm_xdmcp_handle_query"); 
 	return;
     }
     
@@ -425,7 +435,8 @@ gdm_xdmcp_handle_query (struct sockaddr_in *clnt_sa, gint len, gint type)
     }
     
     if (len != explen) {
-	gdm_error (_("gdm_xdmcp_handle_query: Error in checksum")); 
+	gdm_error (_("%s: Error in checksum"),
+		   "gdm_xdmcp_handle_query"); 
 	XdmcpDisposeARRAYofARRAY8 (&clnt_authlist);
 	return;
     }
@@ -668,14 +679,16 @@ gdm_xdmcp_handle_forward_query (struct sockaddr_in *clnt_sa, gint len)
     
     /* Read display address */
     if (! XdmcpReadARRAY8 (&buf, &clnt_addr)) {
-	gdm_error (_("gdm_xdmcp_handle_forward_query: Could not read display address"));
+	gdm_error (_("%s: Could not read display address"),
+		   "gdm_xdmcp_handle_forward_query");
 	return;
     }
     
     /* Read display port */
     if (! XdmcpReadARRAY8 (&buf, &clnt_port)) {
 	XdmcpDisposeARRAY8 (&clnt_addr);
-	gdm_error (_("gdm_xdmcp_handle_forward_query: Could not read display port number"));
+	gdm_error (_("%s: Could not read display port number"),
+		   "gdm_xdmcp_handle_forward_query");
 	return;
     }
     
@@ -683,7 +696,8 @@ gdm_xdmcp_handle_forward_query (struct sockaddr_in *clnt_sa, gint len)
     if (! XdmcpReadARRAYofARRAY8 (&buf, &clnt_authlist)) {
 	XdmcpDisposeARRAY8 (&clnt_addr);
 	XdmcpDisposeARRAY8 (&clnt_port);
-	gdm_error (_("gdm_xdmcp_handle_forward_query: Could not extract authlist from packet")); 
+	gdm_error (_("%s: Could not extract authlist from packet"),
+		   "gdm_xdmcp_handle_forward_query"); 
 	return;
     }
     
@@ -699,13 +713,15 @@ gdm_xdmcp_handle_forward_query (struct sockaddr_in *clnt_sa, gint len)
     }
     
     if (len != explen) {
-	gdm_error (_("gdm_xdmcp_handle_forward_query: Error in checksum")); 
+	gdm_error (_("%s: Error in checksum"),
+		   "gdm_xdmcp_handle_forward_query"); 
 	goto out;
     }
     
     if (clnt_port.length != 2 ||
 	clnt_addr.length != 4) {
-	    gdm_error (_("gdm_xdmcp_handle_forward_query: Bad address")); 
+	    gdm_error (_("%s: Bad address"),
+		       "gdm_xdmcp_handle_forward_query"); 
 	    goto out;
     }
 
@@ -921,25 +937,6 @@ gdm_xdmcp_send_got_managed_forward (struct sockaddr_in *clnt_sa,
 		    (int)sizeof (struct sockaddr_in));
 }
 
-static char *
-get_host_from_addr (struct sockaddr_in *clnt_sa)
-{
-	char *hostname;
-	struct hostent *he;
-
-	/* Find client hostname */
-	he = gethostbyaddr ((gchar *) &(clnt_sa->sin_addr),
-			    sizeof (struct in_addr),
-			    AF_INET);
-
-	if (he != NULL) {
-		hostname = g_strdup (he->h_name);
-	} else {
-		hostname = g_strdup (inet_ntoa (clnt_sa->sin_addr));
-	}
-	return hostname;
-}
-
 static void
 gdm_xdmcp_handle_request (struct sockaddr_in *clnt_sa, gint len)
 {
@@ -959,33 +956,38 @@ gdm_xdmcp_handle_request (struct sockaddr_in *clnt_sa, gint len)
     
     /* Check with tcp_wrappers if client is allowed to access */
     if (! gdm_xdmcp_host_allow (clnt_sa)) {
-	gdm_error (_("gdm_xdmcp_handle_request: Got REQUEST from banned host %s"), 
+	gdm_error (_("%s: Got REQUEST from banned host %s"), 
+		   "gdm_xdmcp_handle_request",
 		   inet_ntoa (clnt_sa->sin_addr));
 	return;
     }
     
     /* Remote display number */
     if (! XdmcpReadCARD16 (&buf, &clnt_dspnum)) {
-	gdm_error (_("gdm_xdmcp_handle_request: Could not read Display Number"));
+	gdm_error (_("%s: Could not read Display Number"),
+		   "gdm_xdmcp_handle_request");
 	return;
     }
     
     /* We don't care about connection type. Address says it all */
     if (! XdmcpReadARRAY16 (&buf, &clnt_conntyp)) {
-	gdm_error (_("gdm_xdmcp_handle_request: Could not read Connection Type"));
+	gdm_error (_("%s: Could not read Connection Type"),
+		   "gdm_xdmcp_handle_request");
 	return;
     }
     
     /* This is TCP/IP - we don't care */
     if (! XdmcpReadARRAYofARRAY8 (&buf, &clnt_addr)) {
-	gdm_error (_("gdm_xdmcp_handle_request: Could not read Client Address"));
+	gdm_error (_("%s: Could not read Client Address"),
+		   "gdm_xdmcp_handle_request");
         XdmcpDisposeARRAY16 (&clnt_conntyp);
 	return;
     }
     
     /* Read authentication type */
     if (! XdmcpReadARRAY8 (&buf, &clnt_authname)) {
-	gdm_error (_("gdm_xdmcp_handle_request: Could not read Authentication Names"));
+	gdm_error (_("%s: Could not read Authentication Names"),
+		   "gdm_xdmcp_handle_request");
 	XdmcpDisposeARRAYofARRAY8 (&clnt_addr);
 	XdmcpDisposeARRAY16 (&clnt_conntyp);
 	return;
@@ -993,7 +995,8 @@ gdm_xdmcp_handle_request (struct sockaddr_in *clnt_sa, gint len)
     
     /* Read authentication data */
     if (! XdmcpReadARRAY8 (&buf, &clnt_authdata)) {
-	gdm_error (_("gdm_xdmcp_handle_request: Could not read Authentication Data"));
+	gdm_error (_("%s: Could not read Authentication Data"),
+		   "gdm_xdmcp_handle_request");
 	XdmcpDisposeARRAYofARRAY8 (&clnt_addr);
 	XdmcpDisposeARRAY16 (&clnt_conntyp);
 	XdmcpDisposeARRAY8 (&clnt_authname);
@@ -1002,7 +1005,8 @@ gdm_xdmcp_handle_request (struct sockaddr_in *clnt_sa, gint len)
     
     /* Read and select from supported authorization list */
     if (! XdmcpReadARRAYofARRAY8 (&buf, &clnt_authorization)) {
-	gdm_error (_("gdm_xdmcp_handle_request: Could not read Authorization List"));
+	gdm_error (_("%s: Could not read Authorization List"),
+		   "gdm_xdmcp_handle_request");
 	XdmcpDisposeARRAY8 (&clnt_authdata);
 	XdmcpDisposeARRAYofARRAY8 (&clnt_addr);
 	XdmcpDisposeARRAY16 (&clnt_conntyp);
@@ -1017,7 +1021,8 @@ gdm_xdmcp_handle_request (struct sockaddr_in *clnt_sa, gint len)
     
     /* Manufacturer ID */
     if (! XdmcpReadARRAY8 (&buf, &clnt_manufacturer)) {
-	gdm_error (_("gdm_xdmcp_handle_request: Could not read Manufacturer ID"));
+	gdm_error (_("%s: Could not read Manufacturer ID"),
+		   "gdm_xdmcp_handle_request");
 	XdmcpDisposeARRAY8 (&clnt_authname);
 	XdmcpDisposeARRAY8 (&clnt_authdata);
 	XdmcpDisposeARRAYofARRAY8 (&clnt_addr);
@@ -1040,7 +1045,8 @@ gdm_xdmcp_handle_request (struct sockaddr_in *clnt_sa, gint len)
     explen += 2+clnt_manufacturer.length;
     
     if (explen != len) {
-	gdm_error (_("gdm_xdmcp_handle_request: Failed checksum from %s"),
+	gdm_error (_("%s: Failed checksum from %s"),
+		   "gdm_xdmcp_handle_request",
 		   inet_ntoa (clnt_sa->sin_addr));
 	XdmcpDisposeARRAY8 (&clnt_authname);
 	XdmcpDisposeARRAY8 (&clnt_authdata);
@@ -1051,42 +1057,46 @@ gdm_xdmcp_handle_request (struct sockaddr_in *clnt_sa, gint len)
 	return;
     }
     
-    gdm_debug ("gdm_xdmcp_handle_request: pending=%d, MaxPending=%d, sessions=%d, MaxSessions=%d",
-	       pending, GdmMaxPending, sessions, GdmMaxSessions);
+    gdm_debug ("gdm_xdmcp_handle_request: xdmcp_pending=%d, MaxPending=%d, xdmcp_sessions=%d, MaxSessions=%d",
+	       xdmcp_pending, GdmMaxPending, xdmcp_sessions, GdmMaxSessions);
 
 
     /* Check if ok to manage display */
     if (mitauth &&
-	sessions < GdmMaxSessions &&
+	xdmcp_sessions < GdmMaxSessions &&
 	(gdm_is_local_addr (&(clnt_sa->sin_addr)) ||
 	 gdm_xdmcp_displays_from_host (&(clnt_sa->sin_addr)) < GdmDispPerHost)) {
 	    char *disp;
-	    char *hostname = get_host_from_addr (clnt_sa);
-	    disp = g_strdup_printf ("%s:%d", hostname, clnt_dspnum);
+	    GdmHostent *he = gdm_gethostbyaddr (&(clnt_sa->sin_addr));
+	    disp = g_strdup_printf ("%s:%d", he->hostname, clnt_dspnum);
 
 	    /* Check if we are already talking to this host */
 	    gdm_xdmcp_display_dispose_check (disp);
 	    g_free (disp);
 
-	    if (pending >= GdmMaxPending) {
+	    if (xdmcp_pending >= GdmMaxPending) {
 		    gdm_debug ("gdm_xdmcp_handle_request: maximum pending");
 		    /* Don't translate, this goes over the wire to servers where we
 		    * don't know the charset or language, so it must be ascii */
 		    gdm_xdmcp_send_decline (clnt_sa, "Maximum pending servers");	
+		    gdm_hostent_free (he);
 	    } else {
-		    gdm_xdmcp_send_accept (hostname, clnt_sa, clnt_dspnum);
+		    /* the addrs are NOT copied */
+		    gdm_xdmcp_send_accept (he /* eaten and freed */, clnt_sa, clnt_dspnum);
 	    }
-
-	    g_free (hostname);
     } else {
 	    /* Don't translate, this goes over the wire to servers where we
 	    * don't know the charset or language, so it must be ascii */
-	    if ( ! mitauth)
+	    if ( ! mitauth) {
 		    gdm_xdmcp_send_decline (clnt_sa, "Only MIT-MAGIC-COOKIE-1 supported");	
-	    else if (sessions >= GdmMaxSessions)
+	    } else if (xdmcp_sessions >= GdmMaxSessions) {
+		    gdm_info ("Maximum number of open XDMCP sessions reached");
 		    gdm_xdmcp_send_decline (clnt_sa, "Maximum number of open sessions reached");	
-	    else 
+	    } else {
+		    gdm_info ("Maximum number of open XDMCP sessions from host %s reached",
+			      inet_ntoa (clnt_sa->sin_addr));
 		    gdm_xdmcp_send_decline (clnt_sa, "Maximum number of open sessions from your host reached");	
+	    }
     }
 
     XdmcpDisposeARRAY8 (&clnt_authname);
@@ -1099,7 +1109,7 @@ gdm_xdmcp_handle_request (struct sockaddr_in *clnt_sa, gint len)
 
 
 static void
-gdm_xdmcp_send_accept (const char *hostname,
+gdm_xdmcp_send_accept (GdmHostent *he /* eaten and freed */,
 		       struct sockaddr_in *clnt_sa,
 		       gint displaynum)
 {
@@ -1110,7 +1120,9 @@ gdm_xdmcp_send_accept (const char *hostname,
     ARRAY8 authdata;
     GdmDisplay *d;
     
-    d = gdm_xdmcp_display_alloc (&(clnt_sa->sin_addr), hostname, displaynum);
+    d = gdm_xdmcp_display_alloc (&(clnt_sa->sin_addr),
+				 he /* eaten and freed */,
+				 displaynum);
     
     authentype.data = (CARD8 *) 0;
     authentype.length = (CARD16) 0;
@@ -1207,20 +1219,23 @@ gdm_xdmcp_handle_manage (struct sockaddr_in *clnt_sa, gint len)
     
     /* Check with tcp_wrappers if client is allowed to access */
     if (! gdm_xdmcp_host_allow (clnt_sa)) {
-	gdm_error (_("gdm_xdmcp_handle_manage: Got Manage from banned host %s"), 
+	gdm_error (_("%s: Got Manage from banned host %s"), 
+		   "gdm_xdmcp_handle_manage",
 		   inet_ntoa (clnt_sa->sin_addr));
 	return;
     }
     
     /* SessionID */
     if (! XdmcpReadCARD32 (&buf, &clnt_sessid)) {
-	gdm_error (_("gdm_xdmcp_handle_manage: Could not read Session ID"));
+	gdm_error (_("%s: Could not read Session ID"),
+		   "gdm_xdmcp_handle_manage");
 	return;
     }
     
     /* Remote display number */
     if (! XdmcpReadCARD16 (&buf, &clnt_dspnum)) {
-	gdm_error (_("gdm_xdmcp_handle_manage: Could not read Display Number"));
+	gdm_error (_("%s: Could not read Display Number"),
+		   "gdm_xdmcp_handle_manage");
 	return;
     }
     
@@ -1229,7 +1244,8 @@ gdm_xdmcp_handle_manage (struct sockaddr_in *clnt_sa, gint len)
     
     /* Display Class */
     if (! XdmcpReadARRAY8 (&buf, &clnt_dspclass)) {
-	gdm_error (_("gdm_xdmcp_handle_manage: Could not read Display Class"));
+	gdm_error (_("%s: Could not read Display Class"),
+		   "gdm_xdmcp_handle_manage");
 	return;
     }
     
@@ -1269,8 +1285,8 @@ gdm_xdmcp_handle_manage (struct sockaddr_in *clnt_sa, gint len)
 	}
 	
 	d->dispstat = XDMCP_MANAGED;
-	sessions++;
-	pending--;
+	xdmcp_sessions++;
+	xdmcp_pending--;
 
 	/* Start greeter/session */
 	if (!gdm_display_manage (d)) {
@@ -1442,20 +1458,23 @@ gdm_xdmcp_handle_keepalive (struct sockaddr_in *clnt_sa, gint len)
     
     /* Check with tcp_wrappers if client is allowed to access */
     if (! gdm_xdmcp_host_allow (clnt_sa)) {
-	gdm_error (_("gdm_xdmcp_handle_keepalive: Got KEEPALIVE from banned host %s"), 
+	gdm_error (_("%s: Got KEEPALIVE from banned host %s"), 
+		   "gdm_xdmcp_handle_keepalive",
 		   inet_ntoa (clnt_sa->sin_addr));
 	return;
     }
     
     /* Remote display number */
     if (! XdmcpReadCARD16 (&buf, &clnt_dspnum)) {
-	gdm_error (_("gdm_xdmcp_handle_keepalive: Could not read Display Number"));
+	gdm_error (_("%s: Could not read Display Number"),
+		   "gdm_xdmcp_handle_keepalive");
 	return;
     }
     
     /* SessionID */
     if (! XdmcpReadCARD32 (&buf, &clnt_sessid)) {
-	gdm_error (_("gdm_xdmcp_handle_keepalive: Could not read Session ID"));
+	gdm_error (_("%s: Could not read Session ID"),
+		   "gdm_xdmcp_handle_keepalive");
 	return;
     }
     
@@ -1492,18 +1511,24 @@ gdm_xdmcp_host_allow (struct sockaddr_in *clnt_sa)
      */
     extern int hosts_ctl (char *daemon, char *client_name, char *client_addr, char *client_user);
 
-    struct hostent *client_he;
-    gchar *client;
+    GdmHostent *client_he;
+    char *client;
+    gboolean ret;
     
     /* Find client hostname */
-    client_he = gethostbyaddr ((gchar *) &clnt_sa->sin_addr,
-			       sizeof (struct in_addr),
-			       AF_INET);
-    
-    client = (client_he != NULL) ? client_he->h_name : NULL;
-    
+    client_he = gdm_gethostbyaddr (&(clnt_sa->sin_addr));
+
+    if (client_he->not_found)
+	    client = "unknown";
+    else
+	    client = client_he->hostname;
+
     /* Check with tcp_wrappers if client is allowed to access */
-    return (hosts_ctl ("gdm", client ? client : "unknown", inet_ntoa (clnt_sa->sin_addr), ""));
+    ret = (hosts_ctl ("gdm", client, inet_ntoa (clnt_sa->sin_addr), ""));
+
+    gdm_hostent_free (client_he);
+
+    return ret;
 #else /* HAVE_TCPWRAPPERS */
     return (TRUE);
 #endif /* HAVE_TCPWRAPPERS */
@@ -1511,7 +1536,9 @@ gdm_xdmcp_host_allow (struct sockaddr_in *clnt_sa)
 
 
 static GdmDisplay *
-gdm_xdmcp_display_alloc (struct in_addr *addr, const char *hostname, gint displaynum)
+gdm_xdmcp_display_alloc (struct in_addr *addr,
+			 GdmHostent *he /* eaten and freed */,
+			 int displaynum)
 {
     GdmDisplay *d = NULL;
     
@@ -1523,7 +1550,6 @@ gdm_xdmcp_display_alloc (struct in_addr *addr, const char *hostname, gint displa
     d->greetpid = 0;
     d->servpid = 0;
     d->servstat = 0;
-    d->sessionid = 0;
     d->sesspid = 0;
     d->slavepid = 0;
     d->type = TYPE_XDMCP;
@@ -1550,11 +1576,20 @@ gdm_xdmcp_display_alloc (struct in_addr *addr, const char *hostname, gint displa
 	    d->timed_login_ok = FALSE;
     }
     
-    d->name = g_strdup_printf ("%s:%d", hostname,
+    d->name = g_strdup_printf ("%s:%d", he->hostname,
 			       displaynum);
-    d->hostname = g_strdup (hostname);
+
     memcpy (&d->addr, addr, sizeof (struct in_addr));
 
+    d->hostname = he->hostname;
+    he->hostname = NULL;
+    d->addrs = he->addrs;
+    he->addrs = NULL;
+    d->addr_count = he->addr_count;
+    he->addr_count = 0;
+
+    gdm_hostent_free (he);
+
     d->slave_notify_fd = -1;
     d->master_notify_fd = -1;
     
@@ -1564,10 +1599,10 @@ gdm_xdmcp_display_alloc (struct in_addr *addr, const char *hostname, gint displa
     
     displays = g_slist_append (displays, d);
     
-    pending++;
+    xdmcp_pending++;
     
-    gdm_debug ("gdm_xdmcp_display_alloc: display=%s, session id=%ld, pending=%d ",
-	       d->name, (long)d->sessionid, pending);
+    gdm_debug ("gdm_xdmcp_display_alloc: display=%s, session id=%ld, xdmcp_pending=%d ",
+	       d->name, (long)d->sessionid, xdmcp_pending);
     
     return (d);
 }
@@ -1659,20 +1694,20 @@ gdm_xdmcp_displays_check (void)
 int
 gdm_xdmcp_init  (void)
 {
-	gdm_error (_("gdm_xdmcp_init: No XDMCP support"));
+	gdm_error (_("%s: No XDMCP support"), "gdm_xdmcp_init");
 	return FALSE;
 }
 
 void
 gdm_xdmcp_run (void)
 {
-	gdm_error (_("gdm_xdmcp_run: No XDMCP support"));
+	gdm_error (_("%s: No XDMCP support"), "gdm_xdmcp_run");
 }
 
 void
 gdm_xdmcp_close (void)
 {
-	gdm_error (_("gdm_xdmcp_close: No XDMCP support"));
+	gdm_error (_("%s: No XDMCP support"), "gdm_xdmcp_close");
 }
 
 #endif /* HAVE_LIBXDMCP */
