@@ -1,3 +1,5 @@
+#include "config.h"
+
 #include <gtk/gtk.h>
 #include <libxml/parser.h>
 #include <string.h>
@@ -8,12 +10,20 @@
 #include <locale.h>
 #include <libgnome/gnome-i18n.h>
 
+#include <vicious.h>
+
+#include "gdmwm.h"
+#include "greeter_configuration.h"
 #include "greeter_parser.h"
 #include "greeter_events.h"
 
+/* FIXME: hack */
+extern GreeterItemInfo *welcome_string_info;
+
 static char *file_search_path = NULL;
 
 GHashTable *item_hash = NULL;
+GList *custom_items = NULL;
 
 static gboolean parse_items (xmlNodePtr       node,
 			     GList          **items_out,
@@ -150,7 +160,7 @@ parse_pos (xmlNodePtr       node,
 	  g_set_error (error,
 		       GREETER_PARSER_ERROR,
 		       GREETER_PARSER_ERROR_BAD_SPEC,
-		       "Bad position specifier%s\n", prop);
+		       "Bad position specifier %s\n", prop);
 	  xmlFree (prop);
 	  return FALSE;
 	}
@@ -172,7 +182,7 @@ parse_pos (xmlNodePtr       node,
 	  g_set_error (error,
 		       GREETER_PARSER_ERROR,
 		       GREETER_PARSER_ERROR_BAD_SPEC,
-		       "Bad position specifier%s\n", prop);
+		       "Bad position specifier %s\n", prop);
 	  xmlFree (prop);
 	  return FALSE;
 	}
@@ -198,7 +208,7 @@ parse_pos (xmlNodePtr       node,
 	      g_set_error (error,
 			   GREETER_PARSER_ERROR,
 			   GREETER_PARSER_ERROR_BAD_SPEC,
-			   "Bad size specifier%s\n", prop);
+			   "Bad size specifier %s\n", prop);
 	      xmlFree (prop);
 	      return FALSE;
 	    }
@@ -225,7 +235,7 @@ parse_pos (xmlNodePtr       node,
 	      g_set_error (error,
 			   GREETER_PARSER_ERROR,
 			   GREETER_PARSER_ERROR_BAD_SPEC,
-			   "Bad size specifier%s\n", prop);
+			   "Bad size specifier %s\n", prop);
 	      xmlFree (prop);
 	      return FALSE;
 	    }
@@ -254,7 +264,7 @@ parse_pos (xmlNodePtr       node,
 	  g_set_error (error,
 		       GREETER_PARSER_ERROR,
 		       GREETER_PARSER_ERROR_BAD_SPEC,
-		       "bad expand spec %s\n", prop);
+		       "Bad expand spec %s\n", prop);
 	  xmlFree (prop);
 	  return FALSE;
 	}
@@ -282,11 +292,18 @@ parse_show (xmlNodePtr       node,
       xmlFree (prop);
     }
 
+  /* Note: subtype is deprecated, use type only */
   prop = xmlGetProp (node, "subtype");
   if (prop != NULL)
     {
-      g_free (info->show_subtype);
-      info->show_subtype = g_strdup (prop);
+      /* code for legacy uses of subtype only, are there any such
+       * themes out there?  The Bluecurve was the one this was made
+       * for and bluecurve is NOT using it. */
+      if (info->show_type == NULL ||
+	  strcmp (info->show_type, "system") == 0) {
+	      g_free (info->show_type);
+	      info->show_type = g_strdup (prop);
+      }
       xmlFree (prop);
     }
   
@@ -386,7 +403,7 @@ parse_box (xmlNodePtr       node,
 	  g_set_error (error,
 		       GREETER_PARSER_ERROR,
 		       GREETER_PARSER_ERROR_BAD_SPEC,
-		       "bad orientation %s\n", prop);
+		       "Bad orientation %s\n", prop);
 	  xmlFree (prop);
 	  return FALSE;
 	}
@@ -410,7 +427,7 @@ parse_box (xmlNodePtr       node,
 	  g_set_error (error,
 		       GREETER_PARSER_ERROR,
 		       GREETER_PARSER_ERROR_BAD_SPEC,
-		       "bad homogenous spec %s\n", prop);
+		       "Bad homogenous spec %s\n", prop);
 	  xmlFree (prop);
 	  return FALSE;
 	}
@@ -840,7 +857,7 @@ parse_state_text (xmlNodePtr node,
 static gint
 is_current_locale (const char *lang)
 {
-  const GList *l = gnome_i18n_get_language_list ("LC_MESSAGES");
+  const GList *l = ve_i18n_get_language_list ("LC_MESSAGES");
   int score = 0;
   
   while (l != NULL)
@@ -878,11 +895,13 @@ parse_translated_text (xmlNodePtr node,
   text = xmlNodeGetContent (node);
   if (text == NULL)
     {
-      g_set_error (error,
-		   GREETER_PARSER_ERROR,
-		   GREETER_PARSER_ERROR_BAD_SPEC,
-		   "No string defined for text node\n");
-      return FALSE;
+      /* This is empty text */
+      *translation_score = score;
+      if (*translated_text)
+        g_free (*translated_text);
+      *translated_text = g_strdup ("");
+  
+      return TRUE;
     }
 
   *translation_score = score;
@@ -895,6 +914,120 @@ parse_translated_text (xmlNodePtr node,
   return TRUE;
 }
 
+/* We pass the same arguments as to translated text, since we'll override it
+ * with translation score */
+static gboolean
+parse_stock (xmlNodePtr node,
+	     GreeterItemInfo *info,
+	     char     **translated_text,
+	     gint      *translation_score,
+	     GError   **error)
+{
+  xmlChar *prop;
+  
+  prop = xmlGetProp (node, "type");
+  if (prop)
+    {
+      if (g_ascii_strcasecmp (prop, "language") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("_Language"));
+	}
+      else if (g_ascii_strcasecmp (prop, "session") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("_Session"));
+	}
+      else if (g_ascii_strcasecmp (prop, "system") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("_Actions"));
+	}
+      else if (g_ascii_strcasecmp (prop, "disconnect") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("D_isconnect"));
+	}
+      else if (g_ascii_strcasecmp (prop, "quit") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("_Quit"));
+	}
+      else if (g_ascii_strcasecmp (prop, "halt") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("Shut_down"));
+	}
+      else if (g_ascii_strcasecmp (prop, "suspend") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("Sus_pend"));
+	}
+      else if (g_ascii_strcasecmp (prop, "reboot") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("_Reboot"));
+	}
+      else if (g_ascii_strcasecmp (prop, "chooser") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("_XDMCP Chooser"));
+	}
+      else if (g_ascii_strcasecmp (prop, "config") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("_Configure"));
+	}
+      else if (g_ascii_strcasecmp (prop, "caps-lock-warning") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("You've got capslock on!"));
+	}
+      else if (g_ascii_strcasecmp (prop, "timed-label") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("User %s will login in %d seconds"));
+	}
+      else if (g_ascii_strcasecmp (prop, "welcome-label") == 0)
+        {
+	  /* FIXME: hack */
+	  welcome_string_info = info;
+
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (GdmWelcome);
+	}
+      /* FIXME: is this actually needed? */
+      else if (g_ascii_strcasecmp (prop, "username-label") == 0)
+        {
+	  g_free (*translated_text);
+	  *translated_text = g_strdup (_("Username:"));
+	}
+      else
+        {
+          g_set_error (error,
+		       GREETER_PARSER_ERROR,
+		       GREETER_PARSER_ERROR_BAD_SPEC,
+		       "Bad stock label type\n");
+	  xmlFree (prop);
+	  return FALSE;
+	}
+
+      /* This is the very very very best "translation" */
+      *translation_score = -1;
+
+      xmlFree (prop);
+
+      return TRUE;
+    }
+  else
+    {
+      g_set_error (error,
+		   GREETER_PARSER_ERROR,
+		   GREETER_PARSER_ERROR_BAD_SPEC,
+		   "Stock type not specified\n");
+      return FALSE;
+    }
+}
 
 static gboolean
 parse_label (xmlNodePtr        node,
@@ -903,6 +1036,7 @@ parse_label (xmlNodePtr        node,
 {
   xmlNodePtr child;
   int i;
+  double size_reduction = 1.0;
   char *translated_text;
   gint translation_score = 1000;
   
@@ -932,6 +1066,12 @@ parse_label (xmlNodePtr        node,
 	  if (!parse_translated_text (child, &translated_text, &translation_score, error))
 	    return FALSE;
 	}
+      else if (child->type == XML_ELEMENT_NODE &&
+	       strcmp (child->name, "stock") == 0)
+	{
+	  if (!parse_stock (child, info, &translated_text, &translation_score, error))
+	    return FALSE;
+	}
       else if (strcmp (child->name, "show") == 0)
 	{
 	  if (!parse_show (child, info, error))
@@ -943,7 +1083,7 @@ parse_label (xmlNodePtr        node,
 	  g_set_error (error,
 		       GREETER_PARSER_ERROR,
 		       GREETER_PARSER_ERROR_BAD_SPEC,
-		       "label items cannot have children");
+		       "Label items cannot have children");
 	  return FALSE;
 	}
 	  
@@ -956,6 +1096,15 @@ parse_label (xmlNodePtr        node,
 		   GREETER_PARSER_ERROR,
 		   GREETER_PARSER_ERROR_BAD_SPEC,
 		   "A label must specify the text attribute");
+      return FALSE;
+    }
+  /* FIXME: evil hack to use internally translated strings */
+  if (translation_score == 999 &&
+      ! ve_string_empty (translated_text))
+    {
+      char *foo = g_strdup (_(translated_text));
+      g_free (translated_text);
+      translated_text = foo;
     }
 
   for (i = 0; i < GREETER_ITEM_STATE_MAX; i++)
@@ -974,11 +1123,143 @@ parse_label (xmlNodePtr        node,
   if (info->fonts[GREETER_ITEM_STATE_NORMAL] == NULL)
     info->fonts[GREETER_ITEM_STATE_NORMAL] = pango_font_description_from_string ("Sans");
 
+  if (gdm_wm_screen.width <= 800 &&
+      gdm_wm_screen.width > 640)
+    size_reduction = PANGO_SCALE_SMALL;
+  else if (gdm_wm_screen.width <= 640)
+    size_reduction = PANGO_SCALE_X_SMALL;
+
+  if (size_reduction < 0.99)
+    {
+      for (i = 0; i < GREETER_ITEM_STATE_MAX; i++)
+        {
+          if (info->fonts[i] != NULL)
+	    {
+	      int old_size = pango_font_description_get_size (info->fonts[i]);
+	      pango_font_description_set_size (info->fonts[i], old_size * size_reduction);
+	    }
+	}
+    }
+
   info->orig_text = translated_text;
   
   return TRUE;
 }
 
+static gboolean
+parse_listitem (xmlNodePtr        node,
+		GreeterItemInfo  *info,
+		GError         **error)
+{
+  xmlNodePtr child;
+  char *translated_text = NULL;
+  gint translation_score = 1000;
+  xmlChar *prop;
+  GreeterItemListItem *li;
+  
+  prop = xmlGetProp (node, "id");
+  
+  if (prop)
+    {
+      li = g_new0 (GreeterItemListItem, 1);
+      li->id = g_strdup (prop);
+      xmlFree (prop);
+    }
+  else
+    {
+      g_set_error (error,
+		   GREETER_PARSER_ERROR,
+		   GREETER_PARSER_ERROR_BAD_SPEC,
+		   "Listitem id not specified");
+      return FALSE;
+    }
+
+  child = node->children;
+  while (child)
+    {
+      if (child->type == XML_ELEMENT_NODE &&
+	  strcmp (child->name, "text") == 0)
+	{
+	  if ( ! parse_translated_text (child, &translated_text, &translation_score, error))
+	    {
+              g_free (li->id);
+              g_free (li);
+	      return FALSE;
+	    }
+	}
+    
+      child = child->next;
+    }
+
+  if (translated_text == NULL)
+    {
+      g_free (li->id);
+      g_free (li);
+      g_set_error (error,
+		   GREETER_PARSER_ERROR,
+		   GREETER_PARSER_ERROR_BAD_SPEC,
+		   "A list item must specify the text attribute");
+      return FALSE;
+    }
+  li->text = translated_text;
+
+  info->list_items = g_list_append (info->list_items, li);
+
+  return TRUE;
+}
+
+static gboolean
+parse_list (xmlNodePtr        node,
+	     GreeterItemInfo  *info,
+	     GError         **error)
+{
+  xmlNodePtr child;
+
+  child = node->children;
+  while (child)
+    {
+      if (strcmp (child->name, "pos") == 0)
+	{
+	  if (!parse_pos (child, info, error))
+	    return FALSE;
+	}
+      else if (strcmp (child->name, "show") == 0)
+	{
+	  if (!parse_show (child, info, error))
+	    return FALSE;
+	}
+      else if (strcmp (child->name, "listitem") == 0)
+	{
+	  if ( ! parse_listitem (child, info, error))
+	    return FALSE;
+	}
+      else if (strcmp (child->name, "fixed") == 0 ||
+	       strcmp (child->name, "box") == 0)
+	{
+	  g_set_error (error,
+		       GREETER_PARSER_ERROR,
+		       GREETER_PARSER_ERROR_BAD_SPEC,
+		       "List items cannot have children");
+	  return FALSE;
+	}
+    
+      child = child->next;
+    }
+
+  if (info->list_items != NULL) {
+    if (strcmp (info->id, "userlist") == 0) {
+      g_set_error (error,
+		   GREETER_PARSER_ERROR,
+		   GREETER_PARSER_ERROR_BAD_SPEC,
+		   "List of id userlist cannot have custom list items");
+      return FALSE;
+    }
+    custom_items = g_list_append (custom_items, info);
+  }
+
+  return TRUE;
+}
+
 static gboolean
 parse_entry (xmlNodePtr        node,
 	     GreeterItemInfo  *info,
@@ -1005,7 +1286,7 @@ parse_entry (xmlNodePtr        node,
 	  g_set_error (error,
 		       GREETER_PARSER_ERROR,
 		       GREETER_PARSER_ERROR_BAD_SPEC,
-		       "entry items cannot have children");
+		       "Entry items cannot have children");
 	  return FALSE;
 	}
     
@@ -1027,6 +1308,8 @@ parse_items (xmlNodePtr  node,
     xmlChar *type;
     GreeterItemInfo *info;
     GreeterItemType item_type;
+
+    *items_out = NULL;
     
     items = NULL;
     
@@ -1040,7 +1323,7 @@ parse_items (xmlNodePtr  node,
 		g_set_error (error,
 			     GREETER_PARSER_ERROR,
 			     GREETER_PARSER_ERROR_BAD_SPEC,
-			     "found tag %s when looking for item", child->name);
+			     "Found tag %s when looking for item", child->name);
 		return FALSE;
 	      }
 	    
@@ -1050,7 +1333,7 @@ parse_items (xmlNodePtr  node,
 		g_set_error (error,
 			     GREETER_PARSER_ERROR,
 			     GREETER_PARSER_ERROR_BAD_SPEC,
-			     "items must specify their type");
+			     "Items must specify their type");
 		return FALSE;
 	      }
 
@@ -1064,12 +1347,14 @@ parse_items (xmlNodePtr  node,
 	      item_type = GREETER_ITEM_TYPE_LABEL;
 	    else if (strcmp (type, "entry") == 0)
 	      item_type = GREETER_ITEM_TYPE_ENTRY;
+	    else if (strcmp (type, "list") == 0)
+	      item_type = GREETER_ITEM_TYPE_LIST;
 	    else
 	      {
 		g_set_error (error,
 			     GREETER_PARSER_ERROR,
 			     GREETER_PARSER_ERROR_BAD_SPEC,
-			     "unknown item type %s", type);
+			     "Unknown item type %s", type);
 		xmlFree (type);
 		return FALSE;
 	      }
@@ -1097,11 +1382,14 @@ parse_items (xmlNodePtr  node,
 	      case GREETER_ITEM_TYPE_ENTRY:
 		res = parse_entry (child, info, error);
 		break;
+	      case GREETER_ITEM_TYPE_LIST:
+		res = parse_list (child, info, error);
+		break;
 	      default:
 		g_set_error (error,
 			     GREETER_PARSER_ERROR,
 			     GREETER_PARSER_ERROR_BAD_SPEC,
-			     "bad item type");
+			     "Bad item type");
 		res = FALSE;
 	      }
 	    
@@ -1143,6 +1431,7 @@ greeter_parse (const char *file, const char *datadir,
   GList *items;
 
   /* FIXME: EVIL! GLOBAL! */
+  g_free (file_search_path);
   file_search_path = g_strdup (datadir);
   
   if (!g_file_test (file, G_FILE_TEST_EXISTS))
@@ -1168,6 +1457,7 @@ greeter_parse (const char *file, const char *datadir,
   node = xmlDocGetRootElement (doc);
   if (node == NULL)
     {
+      xmlFreeDoc (doc);
       g_set_error (error,
 		   GREETER_PARSER_ERROR,
 		   GREETER_PARSER_ERROR_BAD_XML,
@@ -1177,6 +1467,7 @@ greeter_parse (const char *file, const char *datadir,
   
   if (strcmp (node->name, "greeter") != 0)
     {
+      xmlFreeDoc (doc);
       g_set_error (error,
 		   GREETER_PARSER_ERROR,
 		   GREETER_PARSER_ERROR_WRONG_TYPE,
@@ -1185,21 +1476,34 @@ greeter_parse (const char *file, const char *datadir,
     }
 
 
-  item_hash = g_hash_table_new_full ((GHashFunc)greeter_info_id_hash,
-				     (GEqualFunc)greeter_info_id_equal,
-				     NULL,
-				     (GDestroyNotify)greeter_item_info_free);
+  item_hash = g_hash_table_new ((GHashFunc)greeter_info_id_hash,
+				(GEqualFunc)greeter_info_id_equal);
   
 
   root = greeter_item_info_new (NULL, GREETER_ITEM_TYPE_RECT);
   res = parse_items (node, &items, root, error);
   if (!res)
     {
+      welcome_string_info = NULL;
+
+      g_hash_table_destroy (item_hash);
+      item_hash = NULL;
+      g_list_free (custom_items);
+      custom_items = NULL;
+
+      g_list_foreach (items, (GFunc) greeter_item_info_free, NULL);
+      g_list_free (items);
+      items = NULL;
+
       greeter_item_info_free (root);
 
+      xmlFreeDoc (doc);
+
       return NULL;
     }
 
+  xmlFreeDoc (doc);
+
   root->fixed_children = items;
   
   root->x = 0;
@@ -1216,3 +1520,9 @@ greeter_parse (const char *file, const char *datadir,
   
   return root;
 }
+
+const GList *
+greeter_custom_items (void)
+{
+  return custom_items;
+}
